@@ -1,6 +1,11 @@
+#![allow(clippy::needless_update)]
+
+use std::sync::Arc;
+
 use rayon::prelude::*;
-use swc_common::{pass::Repeated, util::take::Take, DUMMY_SP};
+use swc_common::{collections::AHashSet, pass::Repeated, util::take::Take, DUMMY_SP, GLOBALS};
 use swc_ecma_ast::*;
+use swc_ecma_utils::collect_decls;
 use swc_ecma_visit::{noop_visit_mut_type, VisitMut, VisitMutWith, VisitWith};
 use tracing::{span, Level};
 
@@ -49,6 +54,7 @@ where
         enable_everything,
         mode,
         debug_infinite_loop,
+        bindings: Default::default(),
     }
 }
 
@@ -62,6 +68,8 @@ struct Pure<'a, M> {
     mode: &'a M,
 
     debug_infinite_loop: bool,
+
+    bindings: Option<Arc<AHashSet<Id>>>,
 }
 
 impl<M> Repeated for Pure<'_, M> {
@@ -70,6 +78,7 @@ impl<M> Repeated for Pure<'_, M> {
     }
 
     fn reset(&mut self) {
+        self.bindings = None;
         self.ctx = Default::default();
         self.changed = false;
     }
@@ -157,34 +166,40 @@ where
                     enable_everything: self.enable_everything,
                     mode: self.mode,
                     debug_infinite_loop: self.debug_infinite_loop,
+                    bindings: self.bindings.clone(),
                 };
                 node.visit_mut_with(&mut v);
 
                 self.changed |= v.changed;
             }
         } else {
-            let changed = nodes
-                .par_iter_mut()
-                .map(|node| {
-                    let mut v = Pure {
-                        options: self.options,
-                        marks: self.marks,
-                        ctx: Ctx {
-                            par_depth: self.ctx.par_depth + 1,
-                            ..self.ctx
-                        },
-                        changed: false,
-                        enable_everything: self.enable_everything,
-                        mode: self.mode,
-                        debug_infinite_loop: self.debug_infinite_loop,
-                    };
-                    node.visit_mut_with(&mut v);
+            GLOBALS.with(|globals| {
+                let changed = nodes
+                    .par_iter_mut()
+                    .map(|node| {
+                        GLOBALS.set(globals, || {
+                            let mut v = Pure {
+                                options: self.options,
+                                marks: self.marks,
+                                ctx: Ctx {
+                                    par_depth: self.ctx.par_depth + 1,
+                                    ..self.ctx
+                                },
+                                changed: false,
+                                enable_everything: self.enable_everything,
+                                mode: self.mode,
+                                debug_infinite_loop: self.debug_infinite_loop,
+                                bindings: self.bindings.clone(),
+                            };
+                            node.visit_mut_with(&mut v);
 
-                    v.changed
-                })
-                .reduce(|| false, |a, b| a || b);
+                            v.changed
+                        })
+                    })
+                    .reduce(|| false, |a, b| a || b);
 
-            self.changed |= changed;
+                self.changed |= changed;
+            });
         }
     }
 }
@@ -413,6 +428,8 @@ where
     }
 
     fn visit_mut_module_items(&mut self, items: &mut Vec<ModuleItem>) {
+        self.bindings = Some(Arc::new(collect_decls(items)));
+
         self.visit_par(items);
 
         self.handle_stmt_likes(items);
@@ -430,6 +447,26 @@ where
         e.args.visit_mut_with(self);
     }
 
+    fn visit_mut_opt_var_decl_or_expr(&mut self, n: &mut Option<VarDeclOrExpr>) {
+        n.visit_mut_children_with(self);
+
+        if self.options.side_effects {
+            if let Some(VarDeclOrExpr::Expr(e)) = n {
+                self.ignore_return_value(
+                    e,
+                    DropOpts {
+                        drop_zero: true,
+                        drop_str_lit: true,
+                        ..Default::default()
+                    },
+                );
+                if e.is_invalid() {
+                    *n = None;
+                }
+            }
+        }
+    }
+
     fn visit_mut_pat_or_expr(&mut self, n: &mut PatOrExpr) {
         n.visit_mut_children_with(self);
 
